<?
/*
full list of items that can be passed to check for spam / submit spam / submit ham
$msgInfo = array(
	'user_ip' => '',
	'user_agent' => '',
	'referrer' => '',
	'permalink' => '',
	'comment_type' => '',
	'comment_author' => '',
	'comment_author_email' => '',
	'comment_author_url' => '',
	'comment_content' => ''
);

sample usage:
$akismet = new Gregphoto_Service_Akismet('xxxxxx','http://www.gregphoto.net');
if($akismet->checkIsSpam($msgInfo)) {
	// it is spam
} else {
	// it is not spam
}
*/

/**
 * A class for dealing with the Akismet web service.  Akismet provides a service to help people identify messages - specially blog comments - as spam or not spam.
 *
 */
class Gregphoto_Service_Akismet {
	protected $key;
	protected $blog;
	protected $userAgent = 'Gregphoto_Service_Akismet/1.0 | Gregphoto_Service_Akismet/1.0';
	protected $vars = array();
	protected $validVars = array('key','blog','user_ip','user_agent','referrer','permalink','comment_type','comment_author','comment_author_email','comment_author_url','comment_content');
	
	/**
	 * Constructor for the class, must be called
	 *
	 * @param string $key An API key for the Akismet web service.  Instructions for getting one are available at http://akismet.com/development/api/
	 * @param string $blog The main page of the website / blog this is stopping spam for
	 */
	public function __construct($key,$blog) {
		if(!isset($key) || $key == '') {
			throw new Zend_Service_Exception("You must include the Akismet key.  See http://www.akismet.com for more details.");
		}
		if(!isset($blog) || $blog == '') {
			throw new Zend_Service_Exception("You must include the URL to the first page of your website/blog");
		}
		$this->key = $key;
		$this->blog = $blog;
		$this->reset();
	}
	
	/**
	 * Validates whether or not your API key is valid
	 *
	 * @return Boolean Returns true if key is valid, false if invalid
	 */
	public function validateKey() {
		$url = 'http://rest.akismet.com/1.1/verify-key';
		if($this->sendReq($url) == 'valid') return true;
		return false;
	}
	
	/**
	 * Checks whether a comment / message is spam
	 *
	 * @param array $msgInfo An array of items to be passed as Post vars to the Akismet web service.  By default, user_ip, user_agent, and referrer are set using the $_SERVER variable.  You may override these as well.
	 * @return Boolean Returns true is the comment is identified as spam, false if it is not
	 */
	public function checkIsSpam($msgInfo) {
		$url = "http://{$this->key}.rest.akismet.com/1.1/comment-check";
		$this->vars = array_merge($this->vars,$this->getDefaultVars(),$msgInfo);
		if($this->sendReq($url) == 'true') return true;
		return false;
	}
	
	/**
	 * This call is for submitting comments as spam that weren't identified as spam by checkIsSpam. It takes identical arguments as checkIsSpam.
	 *
	 * @param array $msgInfo An array of items to be passed as Post vars to the Akismet web service.  By default, user_ip, user_agent, and referrer are set using the $_SERVER variable.  You may override these as well.
	 */
	public function submitSpam($msgInfo) {
		$url = "http://{$this->key}.rest.akismet.com/1.1/submit-spam";
		$this->vars = array_merge($this->vars,$this->getDefaultVars(),$msgInfo);
		$this->sendReq($url);
	}

	/**
	 * This call is for submitting comments as ham (not spam) that were identified as spam by checkIsSpam. It takes identical arguments as checkIsSpam.
	 *
	 * @param array $msgInfo An array of items to be passed as Post vars to the Akismet web service.  By default, user_ip, user_agent, and referrer are set using the $_SERVER variable.  You may override these as well.
	 */
	public function submitHam($msgInfo) {
		$url = "http://{$this->key}.rest.akismet.com/1.1/submit-ham";
		$this->vars = array_merge($this->vars,$this->getDefaultVars(),$msgInfo);
		$this->sendReq($url);
	}
	
	/**
	 * Resets the class variable vars to only contain entries for key and blog.  Sets up the class for further requests.
	 *
	 */
	protected function reset() {
		$this->vars = array('key' => $this->key, 'blog' => $this->blog);
	}
	
	/**
	 * Performs the HTTP request to the Akismet web service, adds the POST requests, and returns the response body from Akismet as a string
	 *
	 * @param string $url The url to send the request to
	 * @return string The body of the HTTP response from Akismet
	 */
	protected function sendReq($url) {
		$http = new Zend_Http_Client($url);
		$http->setUserAgent($this->userAgent);
		foreach($this->vars as $key=>$value) {
			if(in_array($key,$this->validVars) || array_key_exists($key,$_SERVER)) {
				$http->setParameterPost($key,$value);
			}
		}
		$req = $http->request(Zend_Http_Client::METHOD_POST);
		$this->reset();
		return $req->getBody();
	}
	
	/**
	 * Sets some default values for the user_ip, user_agent, and referrer options from the $_SERVER superglobal
	 *
	 * @return array Array of default options
	 */
	protected function getDefaultVars() {
		return array(
			'user_ip' => $_SERVER['REMOTE_ADDR'],
			'user_agent' => $_SERVER['HTTP_USER_AGENT'],
			'referrer' => $_SERVER['HTTP_REFERER']
		);
	}
}